import datetime
import shutil
import urllib.parse
from pathlib import Path
from typing import List, Optional

import typer
from requests import request

from sprint_velocity.plot_matplolib import generate_plot
from sprint_velocity.utils import (
    DATETIME_HELP,
    complete_date_format,
    display_settings,
    get_current_sprint_info,
    get_header,
    get_json_data,
    get_project,
    save_json_data,
)

app = typer.Typer()


@app.command()
def settings_backupfile(
    backup_file_path: Path = typer.Argument(
        ..., show_default=False, help="The path to your backup folder."
    ),
    date_format: str = typer.Argument(
        help=DATETIME_HELP, default="%Y-%m-%d_%H_%M_%S", autocompletion=complete_date_format
    ),
):
    json_data = get_json_data()
    json_data["backup_file_path"] = backup_file_path.as_posix()
    json_data["date_format"] = date_format
    save_json_data(json_data)


@app.command()
def settings_filestorage(
    file_path: Path = typer.Argument(
        ..., show_default=False, help="The path where your files should be stored."
    )
):
    json_data = get_json_data()
    json_data["file_path"] = file_path.as_posix()
    save_json_data(json_data)


@app.command()
def settings_board(
    company: str = typer.Argument(
        ..., show_default=False, help="The name of the company the board is from. Example: adesso"
    ),
    board_id: int = typer.Argument(
        ...,
        show_default=False,
        help="The id of the board. "
        "You can find these inside of the url "
        "`https://jira.foobar.de/secure/RapidBoard.jspa?rapidView=1234` "
        "it is the value of the `rapidView` => 1234.",
    ),
    token: str = typer.Argument(
        ...,
        show_default=False,
        help="The token can be generated in your profile section inside your jira.",
    ),
    url: str = typer.Argument(
        ..., show_default=False, help="The url of your board/jira server. https://jira.foobar.de"
    ),
):
    json_data = get_json_data()
    json_data[company] = {board_id: {"token": token, "url": f"{url}/rest"}}
    save_json_data(json_data)


@app.command()
def display_config():
    json_data = get_json_data()
    display_settings(json_data)


@app.command()
def velocity_graph(
    company: str = typer.Argument(
        ..., show_default=False, help="The company name you used in settings."
    ),
    board_ids: Optional[List[int]] = typer.Argument(
        None,
        show_default=False,
        help="If specified it will only create plots for the selected board_ids "
        "otherwise all board_ids belonging to the selected company will be created.",
    ),
):
    json_data = get_json_data()
    if not (company_data := json_data.get(company)):
        typer.secho(
            f"Please create settings for {company} via `jira_statistics settings-board --help`.",
            fg=typer.colors.RED,
        )
        raise typer.Abort

    ids = board_ids if board_ids else []

    for board_id in company_data:
        if ids and int(board_id) not in ids:
            continue

        int_board_id = int(board_id)
        token = company_data[board_id]["token"]
        url = company_data[board_id]["url"]

        headers = get_header(token)
        project = get_project(url, int_board_id, headers)
        sprint_info = get_current_sprint_info(url, int_board_id, headers)
        sprint_start_date = sprint_info["start_date"]

        search_str = f"project = {project} and issuetype in subTaskIssueTypes() AND Sprint = {sprint_info['id']} AND (resolution = unresolved or resolved >= {sprint_start_date})"  # noqa: E501
        search_query = urllib.parse.quote(search_str)

        base_url = f"{url}/api/2/search?jql={search_query}"
        response = request("GET", url=base_url, headers=headers)

        file_root = Path(json_data.get("file_path", ""), project)
        if not json_data.get("file_path"):
            typer.secho(
                "For a custom file path run `jira_statistics settings-filestorage --help`.",
                fg=typer.colors.BRIGHT_MAGENTA,
            )
        outputfile = file_root / Path(sprint_info["name"])
        if not file_root.exists():
            file_root.mkdir(parents=True)

        if backup_file_path := json_data.get("backup_file_path"):

            new_file = Path(f"{outputfile}.png")
            if new_file.exists():
                if not Path(backup_file_path).exists():
                    Path(backup_file_path).mkdir(parents=True)

                filename = outputfile.name
                date_str = datetime.datetime.now().strftime(
                    json_data.get("date_format", "%Y-%m-%d")
                )
                backup_file = Path(backup_file_path) / Path(f"{filename}_{date_str}.png")
                shutil.copy(new_file, backup_file)

        generate_plot(response.json(), sprint_start_date, outputfile)

        typer.echo(f"Plot was saved under {outputfile}.png")

    typer.echo("All plots processed.")


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sprint_velocity']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib', 'pandas', 'pendulum', 'requests', 'strongtyping', 'typer[all]']

entry_points = \
{'console_scripts': ['jira_statistics = sprint_velocity.main:app']}

setup_kwargs = {
    'name': 'sprint-velocity',
    'version': '1.0.5',
    'description': 'Generating a Matplotlib plot to see the scrum velocity for a sprint.',
    'long_description': "![Python application](https://github.com/FelixTheC/jira_srcum_velocity/workflows/Python%20application/badge.svg)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Imports: isort](https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336)](https://pycqa.github.io/isort/)\n\n# Jira Sprint Velocity\n- generate a Sprint Velocity Matplotlib Graph\n\n## How To Use\n```shell\n$ jira_statistics --help\n\n```\n\n## Used Jira Filter\n- to get all issues for generating the Velocity we're using following Statement\n```shell\nproject = <project> and issuetype in subTaskIssueTypes() AND Sprint = <sprint_id> AND (resolution = unresolved or resolved >= <sprint_start_date>)\n```\n\n## Authentication\n- the rest api from jira uses a Bearer Token, this Token can be created directly in Jira under Profile\n\n\n## Example Result\n![SprintVelocity](images/test.png)\n",
    'author': 'FelixTheC',
    'author_email': 'fberndt87@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FelixTheC/jira_srcum_velocity',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import datetime
import json
from json import JSONDecodeError
from pathlib import Path

import pendulum
import typer
from requests import request
from requests.structures import CaseInsensitiveDict
from strongtyping.strong_typing import match_typing

BASE_CONFIG_FILE = "config.json"
CONFIG_FILE_PATH = Path(__file__).parent / BASE_CONFIG_FILE

DATETIME_HELP = """
%a => Weekday as locale’s abbreviated name.

%A => Weekday as locale’s full name.

%w => Weekday as a decimal number, where 0 is Sunday and 6 is Saturday.

%d => Day of the month as a zero-padded decimal number.

%b => Month as locale’s abbreviated name.

%B => Month as locale’s full name.

%m => Month as a zero-padded decimal number.

%y => Year without century as a zero-padded decimal number.

%Y => Year with century as a decimal number.

%H => Hour (24-hour clock) as a zero-padded decimal number.

%I => Hour (12-hour clock) as a zero-padded decimal number.

%p => Locale’s equivalent of either AM or PM.

%M => Minute as a zero-padded decimal number.

%S => Second as a zero-padded decimal number.

%f => Microsecond as a decimal number, zero-padded to 6 digits.

%z => UTC offset in the form ±HHMM[SS[.ffffff]] (empty string if the object is naive).

%Z => Time zone name (empty string if the object is naive).

%j => Day of the year as a zero-padded decimal number.

%U => Week number of the year (Sunday as the first day of the week) as a zero-padded decimal number.
All days in a new year preceding the first Sunday are considered to be in week 0.

%W => Week number of the year (Monday as the first day of the week) as a zero-padded decimal number.
All days in a new year preceding the first Monday are considered to be in week 0.

%c => Locale’s appropriate date and time representation.

%x => Locale’s appropriate date representation.

%X => Locale’s appropriate time representation.
"""


@match_typing
def create_dates(start_end: datetime.date, end_date: datetime.date):
    period = pendulum.period(start=start_end, end=end_date)
    for dt in period.range("days"):
        if dt.weekday() not in (5, 6):
            yield dt


def current_date():
    return datetime.date.today()


def get_json_data() -> dict:
    if not CONFIG_FILE_PATH.exists():
        return {}
    with CONFIG_FILE_PATH.open("r") as file:
        try:
            return json.load(file)
        except JSONDecodeError:
            return {}


@match_typing
def save_json_data(json_data: dict):
    with CONFIG_FILE_PATH.open("w") as file:
        json.dump(json_data, file)


@match_typing
def get_header(token: str) -> CaseInsensitiveDict:
    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    if not token:
        if token_ := get_json_data().get("token"):
            headers["Authorization"] = f"Bearer {token_}"
        else:
            typer.secho("No Token in settings!!!", fg=typer.colors.RED)
            typer.secho(
                "Please add a token to the settings via [settings_token <token>]",
                fg=typer.colors.GREEN,
            )
            raise typer.Abort
    else:
        headers["Authorization"] = f"Bearer {token}"
    return headers


@match_typing
def get_current_sprint_info(url: str, board_id: int, headers: CaseInsensitiveDict):
    base_url = f"{url}/agile/1.0/board/{board_id}/sprint"
    response = request("GET", url=base_url, headers=headers)

    for obj in response.json()["values"]:
        if obj["state"] == "active":
            return {
                "id": obj["id"],
                "start_date": obj["startDate"].split("T")[0],
                "name": obj["name"].lower().replace(" ", "_"),
            }


@match_typing
def get_project(url: str, board_id: int, headers: CaseInsensitiveDict):
    base_url = f"{url}/agile/1.0/board/{board_id}/project"
    response = request("GET", url=base_url, headers=headers)
    values = response.json()["values"]
    return values[0]["key"]


def display_settings(data: dict, tab_level: int = 0):
    idx = 0
    tab = "\t" * tab_level
    for key, val in data.items():
        color = typer.colors.GREEN if idx % 2 == 0 else typer.colors.CYAN
        if isinstance(val, dict):
            typer.secho(f"{tab}{key}: ", fg=color)
            display_settings(val, tab_level=tab_level + 1)
        else:
            typer.secho(f"{tab}{key}: {val}", fg=color)
        idx += 1


valid_datetime_completions = [
    ("%Y-%m-%d %H:%M:%S", "2022-01-31 12:30:59"),
    ("%Y/%m/%d %H:%M:%S", "2022/01/31 12:30:59"),
    ("%Y_%m_%d_%H:%M:%S", "2022_01_31_12:30:59"),
    ("%Y%m%d%H%M%S", "20220131123059"),
]


def complete_date_format(incomplete: str) -> tuple:
    for val, help_text in valid_datetime_completions:
        if val.startswith(incomplete):
            yield val, help_text
